# -*- coding: iso-8859-1 -*-
"""
	MentalWealth is a MoinMoin theme
	by robin.escalation@ACM.org.
	
	Licensed under GPL.
	Do not remove credit or claim it as your own.

	For more info see the readme.html.
"""
from MoinMoin.action import get_available_actions
from MoinMoin.theme import ThemeBase
from MoinMoin.wikiutil import link_tag as link
from MoinMoin.wikiutil import quoteWikinameURL as quoteURL
from MoinMoin.Page import Page

MOREACTIONS = True

class BuildLinks(object):
	"""
		Accumulates a list of links.
	"""
	def __init__(self, request, page):
		self.request = request
		self.url = quoteURL(page.page_name)
		
		self.links = []
			
	def addplain(self, text):
		self.links += [text]
		
	def add(self, action, label):
		url = '%s?action=%s' % (self.url, action)
		txt = self.request.getText(label, formatted=False)
		
		self.links += [link(self.request, url, txt)]

	def __call__(self):
		html = u'<ul class="editbar">\n%s\n</ul>\n' %\
			   '\n'.join(['<li>%s</li>' % item for item in self.links if item != ''])

		return html

class Theme(ThemeBase):
	name = 'mentalwealth-purple'

	def editbarmore(self, d):
		"""
			Assemble panel for additional actions from the parameter dictionary.
		"""
		# short circuit
		if not self.shouldShowEditbar(d['page']):
			return ''

		# use cached copy if possible
		cacheKey = 'editbarmore'
		cached = self._cache.get(cacheKey)
		if cached:
			return cached
		
		# optimisations
		_get = self.request.getText
		page = d['page']

		# we can do these actions
		available = get_available_actions(self.request.cfg, page, self.request.user)

		# start building list of links
		builder = BuildLinks(self.request, page)

		# even more actions
		choices = [key for key, value in self.choices]
		for action in available:
			if action not in choices:
				# split label text
				label = Page(self.request, action).split_title(force=1)
				builder.add(action, label.lower())

		# cache for next call
		html = builder()
		self._cache[cacheKey] = html
		return html

	def editbar(self, d):
		"""
			Assemble the page edit bar from the parameter dictionary.
			This is rewritten here to get rid of fugly drop-down menu.
		"""
		# short circuit
		if not self.shouldShowEditbar(d['page']):
			return ''

		# use cached copy if possible
		cacheKey = 'editbar'
		cached = self._cache.get(cacheKey)
		if cached:
			return cached

		# optimisations
		_get = self.request.getText
		page = d['page']
		
		# start building list of links
		builder = BuildLinks(self.request, page)
		
		# parent page
		parent = page.getParentPage()
		if parent:
			builder.addplain(parent.link_to(self.request, _get('Show Parent', formatted=False)))

		# determine which actions we can use
		available = get_available_actions(self.request.cfg, page, self.request.user)
		for action, label in self.choices:
			if action == 'refresh' and not page.canUseCache():
				continue
			if action == 'edit' and not (page.isWritable() and self.request.user.may.write(page.page_name)):
				continue
			if action[0].isupper() and not action in available:
				continue
			builder.add(action, label)

		# delegate this next part so I can stop rewriting code ;-)
		builder.addplain(self.subscribeLink(page))

		# cache for next call
		html = builder()
		self._cache[cacheKey] = html
		return html

	def header(self, d, **kw):
		"""
			Assemble page header, which includes our sidebar.
			
			@param d: parameter dictionary
			@rtype: string
			@return: page header html
		"""
		_ = self.request.getText

		# there are 6 main panels: each one follows this markup
		html = u'<div class="sidepanel"><h1>%s</h1>%s</div>'
		
		# "search" panel hack so I don't have to rewrite searchform()
		searchpanel = self.searchform(d).replace('<input id="titlesearch"',
												 '<br><input id="titlesearch"')

		# explicit list of possible actions and their text labels
		self.choices = [['edit',			'edit'],
						['diff',			'show changes'],
						['info',			'get info'],
						['raw',				'show raw text'],
						['print',			'show print view'],
						['refresh',			'delete cache'],
						['AttachFile',		'attach file'],
						['SpellCheck',		'check spelling'],
						['LikePages',		'show like pages'],
						['LocalSiteMap',	'show local site map'],
						['RenamePage',		'rename page'],
						['DeletePage',		'delete page']
						]
		
		# bundle up all our parts
		parts = [	self.emit_custom_html(self.cfg.page_header1),
					'<div id="header">%s</div>' % self.logo(),
					self.emit_custom_html(self.cfg.page_header2),
					u'<div id="sidebar">',
					html % (_('Search'),     	searchpanel),
					html % (_('Navigation'), 	self.navibar(d)),
					html % (_('Recent'),     	self.trail(d)),
					html % (_('This Page'),  	self.editbar(d))
				]

		if MOREACTIONS:
			parts += [html % (_('More Actions'), self.editbarmore(d))]

		parts += [	html % (_('User'), self.username(d)),
					self.credits(d),
					u'</div>',
					self.msg(d),		
					self.startPage()
				]
		return u'\n'.join(parts)
	
	def footer(self, d, **kw):
		"""
			Assemble page footer
			
			@param d: parameter dictionary
			@keyword ...:...
			@rtype: string
			@return: page footer html
		"""
		parts = [	u'<div id="pagebottom"></div>',
					self.pageinfo(d['page']),
					self.endPage(),
					self.emit_custom_html(self.cfg.page_footer1),
					self.emit_custom_html(self.cfg.page_footer2),
					]
		return u'\n'.join(parts)

def execute(request):
	"""
		Generate and return a theme object.
	"""
	return Theme(request)
